"use strict";

exports.__esModule = true;

var _react = require("react");

var useAsync = function useAsync(props) {
  var counter = 0;
  var isMounted = false;
  var lastArgs = undefined;

  var initialError = props.initialValue instanceof Error ? props.initialValue : undefined;
  var initialData = initialError ? undefined : props.initialValue;

  var _useState = (0, _react.useState)(initialData),
      data = _useState[0],
      setData = _useState[1];

  var _useState2 = (0, _react.useState)(initialError),
      error = _useState2[0],
      setError = _useState2[1];

  var _useState3 = (0, _react.useState)(props.promiseFn ? new Date() : undefined),
      startedAt = _useState3[0],
      setStartedAt = _useState3[1];

  var _useState4 = (0, _react.useState)(props.initialValue ? new Date() : undefined),
      finishedAt = _useState4[0],
      setFinishedAt = _useState4[1];

  var cancel = function cancel() {
    counter++;
    setStartedAt(undefined);
  };

  var start = function start() {
    counter++;
    setStartedAt(new Date());
    setFinishedAt(undefined);
  };

  var end = function end() {
    return setFinishedAt(new Date());
  };

  var handleData = function handleData(data) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};

    if (isMounted) {
      end();
      setData(data);
      setError(undefined);
      callback(data);
    }
    return data;
  };

  var handleError = function handleError(error) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};

    if (isMounted) {
      end();
      setError(error);
      callback(error);
    }
    return error;
  };

  var onResolve = function onResolve(count) {
    return function (data) {
      return count === counter && handleData(data, props.onResolve);
    };
  };
  var onReject = function onReject(count) {
    return function (error) {
      return count === counter && handleError(error, props.onReject);
    };
  };

  var load = function load() {
    if (props.promiseFn) {
      start();
      props.promiseFn(props).then(onResolve(counter), onReject(counter));
    }
  };

  var run = function run() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (props.deferFn) {
      lastArgs = args;
      start();
      return props.deferFn.apply(props, args.concat([props])).then(onResolve(counter), onReject(counter));
    }
  };

  var reload = function reload() {
    return lastArgs ? run.apply(undefined, lastArgs) : load();
  };

  (0, _react.useEffect)(function () {
    isMounted = true;
    return function () {
      return isMounted = false;
    };
  }, []);

  (0, _react.useEffect)(load, [props.promiseFn, props.watch]);

  return (0, _react.useMemo)(function () {
    return {
      isLoading: startedAt && (!finishedAt || finishedAt < startedAt),
      startedAt: startedAt,
      finishedAt: finishedAt,
      data: data,
      error: error,
      cancel: cancel,
      run: run,
      reload: reload,
      setData: handleData,
      setError: handleError,
      initialValue: props.initialValue
    };
  }, [data, error, startedAt, finishedAt]);
};

var unsupported = function unsupported() {
  throw new Error("useAsync requires react@16.7.0 or later");
};

exports.default = _react.useState ? useAsync : unsupported;